
var inquirer = require("inquirer");
var fs = require("fs");
var path = require("path");

var contractsPath = path.join(".", "modules", "contracts");

function addContract() {
	try {
		var filenames = fs.readdirSync(contractsPath);
		inquirer.prompt([
			{
			  type: "input",
				name: "filename",
				message: "Contract file name (without .js)"
			}
		], function (result) {
			var name = result.filename;
			var type = filenames.length + 1;
			var filename = result.filename + ".js";

			var exampleContract = fs.readFileSync(path.join(__dirname, "..", contract-example.js), "utf8");
			exampleContract = exampleContract.replace(/ExampleContract/g, name);
			exampleContract = exampleContract.replace("//self.type = null;", "self.type = " + type);
			fs.writeFileSync(path.join(contractsPath, filename), exampleContract, "utf8");

			console.log("New contract created: " + ("./contracts/" + filename));
			console.log("Updating contracts list");

			var text = fs.readFileSync(path.join(".", "modules.full.json"), "utf8");
			var modules = JSON.parse(text);
			var contractName = "contracts/" + name;
			var dappPathConfig = "./" + path.join(contractsPath, filename);

			modules[contractName] = dappPathConfig;
			modules = JSON.stringify(modules, false, 2);

			fs.writeFileSync(path.join(".", "modules.full.json"), modules, "utf8");
			console.log("Done");
		});
	} catch (e) {
		console.log(e);
	}
}

function deleteContract() {
	inquirer.prompt([
		{
			type: "input",
			name: "filename",
			message: "Contract file name (without .js)"
		}
	], function (result) {
		var name = result.filename;
		var type = filenames.length + 1;
		var filename = result.filename + ".js";

		var contractPath = path.join(contractsPath, filename);
		var exists = fs.existsSync(contractPath);
		if (!exists) {
			return console.log("Contract not found: " + contractPath);
		}
		try {
			fs.unlinkSync(contractPath);
			console.log("Contract removed");
			console.log("Updating contracts list");
			
			var text = fs.readFileSync(path.join(".", "modules.full.json"), "utf8");
			var modules = JSON.parse(text);
			var name = "contracts/" + name;
			delete modules[name];
			modules = JSON.stringify(modules, false, 2);
			fs.writeFileSync(path.join(".", "modules.full.json"), modules, "utf8");
			console.log("Done");
		} catch (e) {
			console.log(e);
		}
	});
}

module.exports = function (program) {
  program
		.command("contract")
		.description("contract operations")
		.option("-a, --add", "add new contract")
		.option("-d, --delete", "delete contract")
		.action(function (options) {
			var exist = fs.exists(contractsPath);
			if (exist) {
				if (options.add) {
					addContract();
				} else if (options.delete) {
					deleteContract();
				} else {
					console.log("'node contract -h' to get help");
				}
			} else {
				return console.log("./modules/contracts path not found, please change directory to your dapp folder");
			}
		});
}